<?php

namespace Drupal\track_da_files\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\User;
use Drupal\Core\Link;
use Database\Query\SelectExtender;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\file\Entity\File;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class TrackDaFilesController.
 *
 * @package Drupal\track_da_files
 */
class TrackDaFilesController extends ControllerBase {

  /**
   * Builds a table which shows number of displays by file.
   */
  public function table($fid = NULL, $pid = NULL) {

    if (!empty($fid) && !empty($pid)) {
      $build['table'] = $this->table_build_file_report($fid, $pid);
      $file = File::load($fid);
      $filename = $file->getFilename();
      $build['table']['#prefix'] = '<p>' . t('Datas for @filename', ['@filename' => $filename]) . '</p>';
      //$build['table_action_form'] = \Drupal::formBuilder()->getForm('\Drupal\track_da_files\Form\TrackDaFilesTableActionForm'/*, $extra*/);
    }
    else {
      $build['table'] = $this->table_build();
      //$build['table_action_form'] = \Drupal::formBuilder()->getForm('\Drupal\track_da_files\Form\TrackDaFilesTableActionForm', 'buildForm');
    }

    return $build;
  }

  /**
   * Builds a table which shows datas for a specific user.
   */
  public function table_user_report($uid) {

    $output = '';

    $account = User::load($uid);

    if ($uid == '0') {
      $output .= '<p>' . t('Datas for anonymous users') . '</p>';
    }
    else {
      $name = $account->get('name')->value;
      $output .= '<p>' . t('Datas for @username', ['@username' => $name]) . '</p>';
    }

    $build = $this->table_build_user_report($uid);
    //$build['form'] = \Drupal::formBuilder()->getForm('\Drupal\track_da_files\Form\TrackDaFilesUserTableActionForm'/*, $extra*/);

    return $build;
  }


  public function table_build() {

    global $base_url;

    // We check avalaible optional datas.
    $displays_datas = \Drupal::config('track_da_files.settings')
      ->get('displays_datas');
    $files_datas = \Drupal::config('track_da_files.settings')
      ->get('files_datas');

    // We prepare main report table header.
    $header = [$this->t('Filename'), t('Counter')];

    // We put displays datas in main report table header.
    foreach ($displays_datas as $data => $value) {
      if (!empty($value)) {
        if ($value == 'total_ips') {
          $header[] = [
            'data' => $this->t('Total ips'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
        elseif ($value == 'average_by_ip') {
          $header[] = [
            'data' => $this->t('Average by ip'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
        elseif ($value == 'last_display') {
          $header[] = [
            'data' => $this->t('Last display'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
      }
    }

    foreach ($files_datas as $data => $value) {
      if (!empty($value)) {
        if ($value == 'timestamp') {
          $header[] = [
            'data' => $this->t('Created'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
        elseif ($value == 'filesize') {
          $header[] = [
            'data' => $this->t('File size'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
        elseif ($value == 'filemime') {
          $header[] = [
            'data' => $this->t('File mime'),
            'field' => $value,
            'sort' => 'DESC',
          ];
        }
      }
    }


    // We prepare fields.
    $fields = ['pid'/*, 'recid'*/];
    foreach ($displays_datas as $key => $value) {

      if ($value == 'average_by_ip') {
        $average_by_ip = $value;
      }
      elseif ($value == 'total_ips') {
        $total_ips = $value;
      }
      elseif ($value == 'last_display') {
        $last_display = $value;
      }
      else {
        $fields[] = $value;
      }
    }

    $fields2 = ['filename'];
    foreach ($files_datas as $key => $value) {
      if ($value) {
        $fields2[] = $value;
      }
    }

    $header[] = ['data' => t('File datas')];

    // We select the datas in database.
    $query = \Drupal::database()->select('track_da_files_paths', 'p');
    $query->addExpression('COUNT(recid)', 'counter');
    $query->addExpression('COUNT(DISTINCT(ip))', 'total_ips');
    $query->addExpression('MAX(time)', 'last_display');
    $query->addExpression('ROUND(COUNT(recid) / COUNT(DISTINCT(ip)))', 'average_by_ip');
    $query->join('track_da_files', 't', 't.pid = p.pid');
    $query->join('file_managed', 'f', 'p.fid = f.fid');
    $query->extend('\Drupal\Core\Database\Query\PagerSelectExtender');
    $query->extend('\Drupal\Core\Database\Query\TableSortExtender');
    $query->fields('f', $fields2);
    $query->fields('t', $fields);
    $query->fields('p', ['fid', 'path']);
    $query->groupBy('p.fid');
    $query->groupBy('t.pid');
    $query->groupBy('f.filename');
    $query->groupBy('p.path');
    $query->groupBy('f.created');
    $query->groupBy('f.filesize');
    $query->groupBy('f.filemime');
    $query->range(0, 20);

    $results = $query->execute();

    $rows = [];

    // We loop into the results to build table rows for the main report table.
    foreach ($results as $key => $record) {

      foreach ($record as $key2 => $row) {

        if ($key2 == 'filename') {

          //$rows[$key][$key2] = l($row, file_create_url($record->path));
          $record_uri = \Drupal::service('file_url_generator')->generateAbsoluteString($record->path);
          $rows[$key][$key2] = Link::fromTextAndUrl($row, Url::fromUri($record_uri));
        }
        elseif (($key2 == 'pid') || ($key2 == 'path')) {
          unset($rows[$key][$key2]);
        }
        elseif (($key2 == 'last_display' && !empty($last_display)) || ($key2 == 'timestamp')) {
          $rows[$key][$key2] = \Drupal::service('date.formatter')
            ->format($row, 'short');
        }
        elseif ($key2 == 'filesize') {
          $rows[$key][$key2] = format_size($row);
        }
        elseif ((($key2 == 'filemime') || ($key2 == 'counter')) || (!empty($total_ips) && $key2 == 'total_ips')) {
          $rows[$key][$key2] = $row;
        }
        elseif ($key2 == 'fid') {
          //$rows[$key][$key2] = l(t('View more'), 'admin/reports/track_da_files/file_report/' . $record->fid . '/' . $record->pid);
          // $file_uri = file_build_uri('admin/reports/track_da_files/file_report/' . $record->fid . '/' . $record->pid);
          $file_url = $base_url . '/admin/reports/track_da_files/file_report/' . $record->fid . '/' . $record->pid;
          $rows[$key][$key2] = Link::fromTextAndUrl(t('View more'), Url::fromUri($file_url));
        }
        elseif ((!empty($average_by_ip)) && ($key2 == 'average_by_ip')) {
          $rows[$key][$key2] = $record->average_by_ip;
        }
      }
    }

    // We reorder rows to match table headers order.
    $rows_indexes_ordered = [
      'filename',
      'counter',
      'total_ips',
      'average_by_ip',
      'last_display',
      'timestamp',
      'filesize',
      'filemime',
      'fid',
    ];
    $rows_ordered = [];

    foreach ($rows as $row => $value) {
      foreach ($value as $key => $value2) {
        $keys[] = $key;
      }
    }

    $rows_final = [];
    $variables = [];

    foreach ($rows as $row) {
      foreach ($rows_indexes_ordered as $index) {
        if (in_array($index, $keys)) {
          $rows_ordered[$index] = $row[$index];
        }
      }
      $rows_final[] = $rows_ordered;
    }

    $build['track_da_files_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows_final,
      '#attributes' => [
        'id' => 'admin-track-da-files',
        'class' => ['admin-track-da-files'],
      ],
      '#attached' => [
        'library' => [
          'track_da_files/track_da_files',
        ],
      ],
    ];

    $build['track_da_files_pager'] = ['#type' => 'pager'];
    return $build['track_da_files_table'];
  }

  /**
   * Build variables used to create file report.
   */
  public function table_build_file_report($fid, $pid) {

    global $base_url;

    // We retrieve configuration datas.
    $single_file_datas = \Drupal::config('track_da_files.settings')
      ->get('single_file_datas');
    $user_report_enabled = \Drupal::config('track_da_files.settings')
      ->get('user_report_enabled');


    // We prepare table header for file specific report.
    $header = [t('Time')];

    // Header datas.
    foreach ($single_file_datas as $data => $value) {
      if (!empty($value)) {
        if ($value == 'uid') {
          $header[] = $this->t('Username');
        }
        elseif ($value == 'id') {
          $header[] = $this->t('Related content');
        }
        elseif ($value == 'referer') {
          $header[] = $this->t('Displayed from');
        }
        elseif ($value == 'browser') {
          $header[] = $this->t('Browser');
        }
        elseif ($value == 'browser_version') {
          $header[] = $this->t('Browser version');
        }
        elseif ($value == 'browser_platform') {
          $header[] = $this->t('Platform');
        }
        elseif ($value == 'ip') {
          $header[] = $this->t('Ip');
        }
      }
    }

    // We build an array with header values to make some verifications.
    foreach ($header as $key => $value) {
      $header_values[] = $value;
    }


    if ($user_report_enabled) {
      $header[] = $this->t('User datas');
    }

    // We prepare fields.
    $fields = ['time'];

    if (isset($single_file_datas['id']) && $single_file_datas['id'] == '0') {
      unset($single_file_datas['id']);
    }

    foreach ($single_file_datas as $key => $value) {
      if (!empty($value)) {
        if ($value == 'id') {
          $fields[] = 'id';
          $fields[] = 'type';
        }
        else {
          $fields[] = $value;
        }
      }
    }

    $query = \Drupal::database()->select('track_da_files', 't');
    $query->extend('\Drupal\Core\Database\Query\PagerSelectExtender');
    $query->extend('\Drupal\Core\Database\Query\TableSortExtender');
    $query->fields('t', $fields);
    $query->condition('pid', $pid);
    $query->range(0, 20);
    $results = $query->execute();
    $rows = [];

    // We loop into the results to build table rows for file report table.
    foreach ($results as $key => $record) {
      foreach ($record as $key2 => $row) {
        if ($key2 == 'time') {
          $rows[$key][$key2] = \Drupal::service('date.formatter')
            ->format($row, 'short');
        }
        elseif ($key2 == 'id') {
          if ($record->type == 'node') {
            $node = Node::load($row);
            $rows[$key][$key2] = Link::fromTextAndUrl($node->getTitle(), Url::fromUri($base_url . '/' . $record->type . '/' . $row));
          }
          elseif ($record->type == 'comment') {
            $comment = Comment::load($row);
            $rows[$key][$key2] = Link::fromTextAndUrl($comment->getsSubject(), Url::fromUri($base_url . '/' . $record->type . '/' . $row), ['fragment' => 'comment-' . $row]);
          }
          else {
            $rows[$key][$key2] = ' ';
          }
        }
        elseif ($key2 == 'type') {
          unset ($rows[$key][$key2]);
        }
        elseif ($key2 == 'uid') {

          $uid = $row;
          $account = User::load($uid);
          $name = $account->get('name')->value;

          if ($user_report_enabled) {
            //if (in_array('Username', $header_values)) {
              if (!empty($name)) {
                $rows[$key][$key2] = $name;
              }
              else {
                $rows[$key][$key2] = t('Anonymous user');
              }
            //}

            $rows[$key]['view_more'] = Link::fromTextAndUrl(t('View more'), Url::fromUri($base_url . '/admin/reports/track_da_files/user_report/' . $uid));

          }
          else {
            $rows[$key][$key2] = $name;
          }
        }
        else {
          $rows[$key][$key2] = $row;
        }
      }
    }
    $build['track_da_files_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => [
        'id' => 'admin-track-da-files',
        'class' => ['admin-track-da-files'],
      ],
      '#attached' => [
        'library' => [
          'track_da_files/track_da_files',
        ],
      ],
    ];

    $build['track_da_files_pager'] = ['#type' => 'pager'];

    return $build['track_da_files_table'];


  }

  /**
   * Build variables used to create table which shows datas for a specific user.
   */
  public function table_build_user_report($uid) {

    $user_report_enabled = \Drupal::config('track_da_files.settings')
      ->get('user_report_enabled');
    $single_user_datas = \Drupal::config('track_da_files.settings')
      ->get('single_user_datas');

    if ($user_report_enabled) {

      $header = [t('Filename'), t('time')];

      // Header datas.
      foreach ($single_user_datas as $data => $value) {
        if (!empty($value)) {
          if ($value == 'referer') {
            $header[] = t('Displayed from');
          }
          elseif ($value == 'browser') {
            $header[] = t('Browser');
          }
          elseif ($value == 'browser_version') {
            $header[] = t('Browser version');
          }
          elseif ($value == 'browser_platform') {
            $header[] = t('Platform');
          }
          elseif ($value == 'ip') {
            $header[] = t('Ip');
          }
        }
      }

      // We prepare fields.
      $fields = ['time'];

      foreach ($single_user_datas as $key => $value) {
        if ($value) {
          $fields[] = $value;
        }
      }

      $query = \Drupal::database()->select('track_da_files', 't');
      $query->join('track_da_files_paths', 'p', 't.pid = p.pid');
      $query->extend('\Drupal\Core\Database\Query\PagerSelectExtender');
      $query->extend('\Drupal\Core\Database\Query\TableSortExtender');
      $query->fields('p', ['fid']);
      $query->fields('t', $fields);
      $query->condition('uid', $uid);
      $query->range(0, 20);
      $results = $query->execute();
      $rows = [];

      // We loop into results to build rows for user report table.
      foreach ($results as $key => $record) {
        foreach ($record as $key2 => $row) {
          if ($key2 == 'time') {
            $rows[$key][$key2] = \Drupal::service('date.formatter')
              ->format($row, 'short');
          }
          elseif ($key2 == 'fid') {

            //$file = file_load($row);
            $file = File::load($row);

            if (isset($file->filename)) {
              $rows[$key][$key2] = $file->getFilename();
            }
            else {
              $rows[$key][$key2] = t('No name');
            }
          }
          else {
            $rows[$key][$key2] = $row;
          }
        }
      }

      $build['track_da_files_table'] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#attributes' => [
          'id' => 'admin-track-da-files',
          'class' => ['admin-track-da-files'],
        ],
        '#attached' => [
          'library' => [
            'track_da_files/track_da_files',
          ],
        ],
      ];

      return $build['track_da_files_table'];
    }
    else {
      return FALSE;
    }

  }

  public function display(Request $request, $scheme = 'public') {
    $test = '';
  }

  public function tracking() {}

  public function tracking_private($file_uri, $uritest) {}

}







