<?php

namespace Drupal\track_da_files\EventSubscriber;

use Drupal\track_da_files\Tdf;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class TdfRequestSubscriber implements EventSubscriberInterface {

  /**
   * @var \Drupal\track_da_files\Tdf
   */
  protected Tdf $tdf;

  /**
   * @param \Drupal\track_da_files\Tdf $tdf
   */
  public function __construct(Tdf $tdf) {
    $this->tdf = $tdf;
  }

  /**
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *
   * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
   */
  public function onRequest(RequestEvent $event) {
    $request = $event->getRequest();
    $path = $request->getPathInfo();
    if (str_starts_with($path , '/system/tdf/')) {
      $tracking = $this->tdf->track($path);
      $headers = [
        'Content-Type' => $tracking['filemime'],
        'Content-Length' => $tracking['filesize'],
      ];
      if (isset($force) && $force == 1) {
        $headers['Content-Disposition'] = 'attachment; filename="' . $tracking['filename'] . '"';
      }
      $response = new BinaryFileResponse($tracking['uri'], 200, $headers);
      $response->send();

      return $response;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onRequest', 99];

    return $events;
  }
}

