<?php

namespace Drupal\track_da_files\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Plugin\Field\FieldFormatter\GenericFileFormatter;

/**
 * Plugin implementation of the 'TDF Generic files' formatter.
 *
 * @FieldFormatter(
 *   id = "tdf_default",
 *   label = @Translation("TDF Generic files"),
 *   field_types = {
 *     "file"
 *   },
 * )
 */
class TdfGenericFileFormatter extends GenericFileFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {
      $item = $file->_referringItem;
      $entity = $items->getEntity();
      $type = $entity->getEntityTypeId();
      $entity_bundle = $entity->getType();
      $id = $entity->id();
      $elements[$delta] = [
        '#theme' => 'track_da_files_file_link',
        '#file' => $file,
        '#description' => $this->getSetting('use_description_as_link_text') ? $item->description : NULL,
        '#entity_type' => $type,
        '#entity_bundle' => $entity_bundle,
        '#entity_id' => $id,
        '#cache' => [
          'tags' => $file->getCacheTags(),
        ],
      ];
      if (isset($item->_attributes)) {
        $elements[$delta] += ['#attributes' => []];
        $elements[$delta]['#attributes'] += $item->_attributes;
        unset($item->_attributes);
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $form['force_download'] = [
      '#title' => t('Force download'),
      '#type' => 'checkbox',
      '#default_value' =>  $this->getSetting('force_download'),
    ];
    $form['custom_link_text'] = [
      '#type' => 'textfield',
      '#title' => t('Custom link text'),
      '#default_value' => $this->getSetting('custom_link_text'),
    ];

    return $form;
  }

}
