<?php

namespace Drupal\track_da_files\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Plugin\Field\FieldFormatter\TableFormatter;

/**
 * Plugin implementation of the 'file_table' formatter.
 *
 * @FieldFormatter(
 *   id = "tdf_table",
 *   label = @Translation("Table of files"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class TdfTableFormatter extends TableFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    if ($files = $this->getEntitiesToView($items, $langcode)) {
      $header = [$this->t('Attachment'), $this->t('Size')];
      $rows = [];
      foreach ($files as $file) {
        $item = $file->_referringItem;
        $entity = $items->getEntity();
        $type = $entity->getEntityTypeId();
        $entity_bundle = $entity->getType();
        $id = $entity->id();
        $rows[] = [
          [
            'data' => [
              '#theme' => 'track_da_files_file_link',
              '#file' => $file,
              '#description' => $this->getSetting('use_description_as_link_text') ? $item->description : NULL,
              '#entity_type' => $type,
              '#entity_bundle' => $entity_bundle,
              '#entity_id' => $id,
              '#cache' => [
                'tags' => $file->getCacheTags(),
              ],
            ],
          ],
          ['data' => format_size($file->getSize())],
        ];
      }

      $elements[0] = [];
      if (!empty($rows)) {
        $elements[0] = [
          '#theme' => 'table__file_formatter_table',
          '#header' => $header,
          '#rows' => $rows,
        ];
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $form['force_download'] = [
      '#title' => t('Force download'),
      '#type' => 'checkbox',
      '#default_value' =>  $this->getSetting('force_download'),
    ];
    $form['custom_link_text'] = [
      '#type' => 'textfield',
      '#title' => t('Custom link text'),
      '#default_value' => $this->getSetting('custom_link_text'),
    ];

    return $form;
  }

}
