<?php

namespace Drupal\track_da_files\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\FileInterface;
use Drupal\file\Plugin\Field\FieldFormatter\UrlPlainFormatter;

/**
 * Plugin implementation of the 'TDF URL to file' formatter.
 *
 * @FieldFormatter(
 *   id = "tdf_url_plain",
 *   label = @Translation("TDF URL to file"),
 *   field_types = {
 *     "file"
 *   },
 * )
 */
class TdfUrlPlainFormatter extends UrlPlainFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {
      assert($file instanceof FileInterface);
      $elements[$delta] = [
        '#markup' => $file->createFileUrl(),
        '#cache' => [
          'tags' => $file->getCacheTags(),
        ],
      ];
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $form['force_download'] = [
      '#title' => t('Force download'),
      '#type' => 'checkbox',
      '#default_value' =>  $this->getSetting('force_download'),
    ];
    $form['custom_link_text'] = [
      '#type' => 'textfield',
      '#title' => t('Custom link text'),
      '#default_value' => $this->getSetting('custom_link_text'),
    ];

    return $form;
  }

}
